@@ -22,6 +22,10 @@ from oslo_serialization import jsonutils
 from gbpclient.common import utils
 
 
+neutronV20.UUID_PATTERN = neutronV20.UUID_PATTERN + (
+    '|auto' + neutronV20.HEX_ELEM + '{32}')
+
+
 def _format_fixed_ips(pt):
     try:
         return '\n'.join([jsonutils.dumps(ip) for ip in pt['fixed_ips']])
@@ -80,6 +84,8 @@ class CreatePolicyTarget(neutronV20.CreateCommand):
         parser.add_argument(
             '--fixed-ip', metavar='subnet_id=SUBNET,ip_address=IP_ADDR',
             action='append',
+            type=n_utils.str2dict_type(optional_keys=['subnet_id',
+                                                      'ip_address']),
             help=_('Desired IP and/or subnet for this Policy Target: '
                    'subnet_id=<nid>,ip_address=<ip>. '
                    'You can repeat this option.'))
@@ -91,12 +97,18 @@ class CreatePolicyTarget(neutronV20.CreateCommand):
         parser.add_argument(
             'name', metavar='NAME',
             help=_('Name of Policy Target to create (required argument)'))
+        parser.add_argument(
+            '--segmentation-labels', type=utils.str2list,
+            help=_('Comma separated list of segmentation labels, each label '
+                   'can be upto 255 characters. This option is currently '
+                   'only available with the APIC backend.'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
 
         neutronV20.update_dict(parsed_args, body[self.resource],
-                               ['name', 'tenant_id', 'description'])
+                               ['name', 'tenant_id', 'description',
+                                'segmentation_labels'])
         if parsed_args.policy_target_group:
             body[self.resource]['policy_target_group_id'] = \
                 neutronV20.find_resourceid_by_name_or_id(
@@ -107,12 +119,8 @@ class CreatePolicyTarget(neutronV20.CreateCommand):
             body[self.resource]['port_id'] = (
                 parsed_args.port_id)
 
-        ips = []
         if parsed_args.fixed_ip:
-            for ip_spec in parsed_args.fixed_ip:
-                ip_dict = n_utils.str2dict(ip_spec)
-                ips.append(ip_dict)
-            body[self.resource]['fixed_ips'] = ips
+            body[self.resource]['fixed_ips'] = parsed_args.fixed_ip
 
         return body
 
@@ -140,24 +148,28 @@ class UpdatePolicyTarget(neutronV20.UpdateCommand):
         parser.add_argument(
             '--fixed-ip', metavar='subnet_id=SUBNET,ip_address=IP_ADDR',
             action='append',
+            type=n_utils.str2dict_type(optional_keys=['subnet_id',
+                                                      'ip_address']),
             help=_('Desired IP and/or subnet for this Policy Target: '
                    'subnet_id=<nid>,ip_address=<ip>. '
                    'You can repeat this option.'))
         parser.add_argument(
             '--fixed_ip', action='append', help=argparse.SUPPRESS)
+        parser.add_argument(
+            '--segmentation-labels', type=utils.str2list,
+            help=_('Comma separated list of segmentation labels, each label '
+                   'can be upto 255 characters. This option is currently '
+                   'only available with the APIC backend.'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
 
         neutronV20.update_dict(parsed_args, body[self.resource],
-                               ['name', 'tenant_id', 'description'])
+                               ['name', 'tenant_id', 'description',
+                                'segmentation_labels'])
 
-        ips = []
         if parsed_args.fixed_ip:
-            for ip_spec in parsed_args.fixed_ip:
-                ip_dict = n_utils.str2dict(ip_spec)
-                ips.append(ip_dict)
-            body[self.resource]['fixed_ips'] = ips
+            body[self.resource]['fixed_ips'] = parsed_args.fixed_ip
 
         return body
 
@@ -213,6 +225,12 @@ class CreatePolicyTargetGroup(neutronV20.CreateCommand):
         n_utils.add_boolean_argument(
             parser, '--shared', dest='shared',
             help=_('Enable or disable resource sharing, default is False'))
+        n_utils.add_boolean_argument(
+            parser, '--intra-ptg-allow', dest='intra_ptg_allow',
+            help=_("Allow or disallow communication between Policy Targets of "
+                   "this Policy Target Group. Default is True, i.e., all "
+                   "Policy Targets can communicate with each other. This "
+                   "option is only available with the APIC backend."))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -248,7 +266,8 @@ class CreatePolicyTargetGroup(neutronV20.CreateCommand):
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
                                 'provided_policy_rule_sets', 'subnets',
-                                'consumed_policy_rule_sets', 'shared'])
+                                'consumed_policy_rule_sets', 'shared',
+                                'intra_ptg_allow'])
 
         return body
 
@@ -291,6 +310,12 @@ class UpdatePolicyTargetGroup(neutronV20.UpdateCommand):
         n_utils.add_boolean_argument(
             parser, '--shared', dest='shared',
             help=_('Enable or disable resource sharing, default is False'))
+        n_utils.add_boolean_argument(
+            parser, '--intra-ptg-allow', dest='intra_ptg_allow',
+            help=_("Allow or disallow communication between Policy Targets of "
+                   "this Policy Target Group. Default is True, i.e., all "
+                   "Policy Targets can communicate with each other. This "
+                   "option is only available with the APIC backend."))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -328,7 +353,8 @@ class UpdatePolicyTargetGroup(neutronV20.UpdateCommand):
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
                                 'provided_policy_rule_sets', 'subnets',
-                                'consumed_policy_rule_sets', 'shared'])
+                                'consumed_policy_rule_sets', 'shared',
+                                'intra_ptg_allow'])
 
         return body
 
@@ -381,6 +407,11 @@ class CreateL2Policy(neutronV20.CreateCommand):
         n_utils.add_boolean_argument(
             parser, '--shared', dest='shared',
             help=_('Enable or disable resource sharing, default is False'))
+        parser.add_argument(
+            '--reuse-bd',
+            default=None,
+            help=_('Name or UUID of L2 Policy whose BridgeDomain should '
+                   'be reused (APIC only)'))
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -396,7 +427,11 @@ class CreateL2Policy(neutronV20.CreateCommand):
         if parsed_args.network:
             body[self.resource]['network_id'] = (
                 parsed_args.network)
-
+        if parsed_args.reuse_bd:
+            body[self.resource]['reuse_bd'] = \
+                neutronV20.find_resourceid_by_name_or_id(
+                    self.get_client(), 'l2_policy',
+                    parsed_args.reuse_bd)
         return body
 
 
@@ -487,6 +522,24 @@ class CreateL3Policy(neutronV20.CreateCommand):
             '--subnet-prefix-length',
             type=int,
             help=_('Subnet prefix length, default is 24'))
+        parser.add_argument(
+            '--address-scope-v4-id',
+            help=_('Neutron Address-scope v4 UUID '
+                   '(if not specified, new Neutron Address-scope is '
+                   'created implicitly based on ip_version)'))
+        parser.add_argument(
+            '--address-scope-v6-id',
+            help=_('Neutron Address-scope v6 UUID '
+                   '(if not specified, new Neutron Address-scope is '
+                   'created implicitly based on ip_version)'))
+        parser.add_argument(
+            '--subnetpools-v4', type=utils.str2list,
+            help=_('Comma separated list of Neutron Subnetpool v4 UUIDs '
+                   'if ip_version and address scope is v4'))
+        parser.add_argument(
+            '--subnetpools-v6', type=utils.str2list,
+            help=_('Comma separated list of Neutron Subnetpool v6 UUIDs '
+                   'if ip_version and address scope is v6'))
         parser.add_argument(
             '--external-segment',
             action='append', dest='external_segments', type=utils.str2dict,
@@ -502,6 +555,10 @@ class CreateL3Policy(neutronV20.CreateCommand):
         parser.add_argument(
             'name', metavar='NAME',
             help=_('Name of L3 policy to create (required argument)'))
+        parser.add_argument(
+            '--allowed-vm-names', type=utils.str2list,
+            help=_('Comma separated list of allowed VM name regexes, each '
+                   'regex can be up to 255 characters.'))
         n_utils.add_boolean_argument(
             parser, '--shared', dest='shared',
             help=_('Enable or disable resource sharing, default is False'))
@@ -526,8 +583,11 @@ class CreateL3Policy(neutronV20.CreateCommand):
 
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
-                                'ip_version', 'ip_pool', 'routers',
-                                'subnet_prefix_length', 'shared'])
+                                'ip_version', 'ip_pool',
+                                'address_scope_v4_id', 'address_scope_v6_id',
+                                'subnetpools_v4', 'subnetpools_v6',
+                                'routers', 'subnet_prefix_length', 'shared',
+                                'allowed_vm_names'])
 
         return body
 
@@ -562,12 +622,24 @@ class UpdateL3Policy(neutronV20.UpdateCommand):
             #        '(this option can be repeated)'))
             help=_('New comma separated list of External Segments'
                    '(this option can be repeated)'))
+        parser.add_argument(
+            '--subnetpools-v4', type=utils.str2list,
+            help=_('New comma separated list of Neutron Subnetpool v4 UUIDs '
+                   'if ip_version and address scope is v4'))
+        parser.add_argument(
+            '--subnetpools-v6', type=utils.str2list,
+            help=_('New comma separated list of Neutron Subnetpool v6 UUIDs '
+                   'if ip_version and address scope is v6'))
         parser.add_argument(
             '--routers', type=utils.str2list,
             help=_('New comma separated list of Neutron Router UUIDs'))
         parser.add_argument(
             '--name',
             help=_('New name of the L3 Policy'))
+        parser.add_argument(
+            '--allowed-vm-names', type=utils.str2list,
+            help=_('Comma separated list of allowed VM name regexes, each '
+                   'regex can be up to 255 characters.'))
         n_utils.add_boolean_argument(
             parser, '--shared', dest='shared',
             help=_('Enable or disable resource sharing'))
@@ -595,7 +667,9 @@ class UpdateL3Policy(neutronV20.UpdateCommand):
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description',
                                 'ip_version', 'ip_pool', 'routers',
-                                'subnet_prefix_length', 'shared'])
+                                'subnetpools_v4', 'subnetpools_v6',
+                                'subnet_prefix_length', 'shared',
+                                'allowed_vm_names'])
 
         return body
 
