@@ -52,7 +52,8 @@ class CLITestV20PolicyTargetGroupJSON(test_cli20.CLITestV20Base):
         provided_prs = "icmp-prs=false,web-prs=true"
         consumed_prs = "ssh-prs=true,ftp-prs=false"
         network_service_policy_id = 'network_service_policy_id'
-        shared = 'True'
+        shared = 'true'
+        intra_ptg_allow = 'false'
         args = [name,
                 '--tenant-id', tenant_id,
                 '--description', description,
@@ -60,7 +61,8 @@ class CLITestV20PolicyTargetGroupJSON(test_cli20.CLITestV20Base):
                 '--provided-policy-rule-sets', provided_prs,
                 '--consumed-policy-rule-sets', consumed_prs,
                 '--network-service-policy-id', network_service_policy_id,
-                '--shared', shared]
+                '--shared', shared,
+                '--intra-ptg-allow', intra_ptg_allow]
         position_names = ['name', 'description', 'l2_policy_id',
                           'provided_policy_rule_sets',
                           'consumed_policy_rule_sets',
@@ -77,7 +79,8 @@ class CLITestV20PolicyTargetGroupJSON(test_cli20.CLITestV20Base):
                            network_service_policy_id]
         self._test_create_resource(resource, cmd, name, my_id, args,
                                    position_names, position_values,
-                                   tenant_id=tenant_id, shared=True)
+                                   tenant_id=tenant_id, shared=shared,
+                                   intra_ptg_allow=intra_ptg_allow)
 
     def test_list_policy_target_groups(self):
         """policy-target-group-list."""
@@ -112,7 +115,8 @@ class CLITestV20PolicyTargetGroupJSON(test_cli20.CLITestV20Base):
         provided_prs = "icmp-prs=false,web-prs=true"
         consumed_prs = "ssh-prs=true,ftp-prs=false"
         network_service_policy_id = 'network_service_policy_id'
-        shared = 'True'
+        shared = 'true'
+        intra_ptg_allow = 'false'
         args = [my_id,
                 '--name', name,
                 '--description', description,
@@ -120,7 +124,8 @@ class CLITestV20PolicyTargetGroupJSON(test_cli20.CLITestV20Base):
                 '--provided-policy-rule-sets', provided_prs,
                 '--consumed-policy-rule-sets', consumed_prs,
                 '--network-service-policy-id', network_service_policy_id,
-                '--shared', shared]
+                '--shared', shared,
+                '--intra-ptg-allow', intra_ptg_allow]
         provided_policy_rule_sets = {
             'icmp-prs': 'false',
             'web-prs': 'true'}
@@ -134,7 +139,8 @@ class CLITestV20PolicyTargetGroupJSON(test_cli20.CLITestV20Base):
             'provided_policy_rule_sets': provided_policy_rule_sets,
             'consumed_policy_rule_sets': consumed_policy_rule_sets,
             'network_service_policy_id': network_service_policy_id,
-            'shared': True
+            'shared': shared,
+            'intra_ptg_allow': intra_ptg_allow
         }
         self._test_update_resource(resource, cmd, my_id, args, params)
 
